@@ -1,14 +1,12 @@
-import os
 import json
 import logging
-
+import os
 from typing import Dict
 
-from seldon_core.metrics import split_image_tag
-
 from jsonschema import validate
 from jsonschema.exceptions import ValidationError
 
+from seldon_core.metrics import split_image_tag
 
 logger = logging.getLogger(__name__)
 
@@ -137,6 +135,7 @@ JSON_SCHEMA = {
         "platform": {"type": "string"},
         "inputs": INPUTS_OUTPUTS_SCHEMA,
         "outputs": INPUTS_OUTPUTS_SCHEMA,
+        "custom": {"type": "object", "additionalProperties": {"type": "string"}},
     },
     "additionalProperties": False,
 }
@@ -156,7 +155,7 @@ def validate_model_metadata(data: Dict) -> Dict:
 
     Raises
     ------
-    SeldonInvalidMetadataError if data cannot be properly validated
+    SeldonInvalidMetadataError: if data cannot be properly validated
 
     Notes
     -----
@@ -175,6 +174,7 @@ def validate_model_metadata(data: Dict) -> Dict:
         "platform": "",
         "inputs": [],
         "outputs": [],
+        "custom": {},
     }
 
     data = {**default_meta, **data}
