@@ -1,34 +1,47 @@
-import os
-import yaml
 import logging
-from seldon_core.utils import (
-    extract_request_parts,
-    construct_response,
-    json_to_seldon_message,
-    seldon_message_to_json,
-    construct_response_json,
-    extract_request_parts_json,
-    extract_feedback_request_parts,
+import os
+from typing import Any, Dict, List, Tuple, Union
+
+import numpy as np
+import yaml
+from google.protobuf import json_format
+
+from seldon_core.flask_utils import SeldonMicroserviceException
+from seldon_core.metadata import SeldonInvalidMetadataError, validate_model_metadata
+from seldon_core.metrics import (
+    AGGREGATE_METRIC_METHOD_TAG,
+    FEEDBACK_METRIC_METHOD_TAG,
+    HEALTH_METRIC_METHOD_TAG,
+    INPUT_TRANSFORM_METRIC_METHOD_TAG,
+    OUTPUT_TRANSFORM_METRIC_METHOD_TAG,
+    PREDICT_METRIC_METHOD_TAG,
+    ROUTER_METRIC_METHOD_TAG,
+    SeldonMetrics,
 )
+from seldon_core.proto import prediction_pb2
 from seldon_core.user_model import (
     INCLUDE_METRICS_IN_CLIENT_RESPONSE,
-    client_predict,
-    client_aggregate,
-    client_route,
-    client_custom_metrics,
-    client_transform_output,
-    client_transform_input,
-    client_send_feedback,
-    client_health_status,
     SeldonNotImplementedError,
+    SeldonResponse,
+    client_aggregate,
+    client_custom_metrics,
+    client_health_status,
+    client_predict,
+    client_route,
+    client_send_feedback,
+    client_transform_input,
+    client_transform_output,
+)
+from seldon_core.utils import (
+    construct_response,
+    construct_response_json,
+    extract_feedback_request_parts,
+    extract_request_parts,
+    extract_request_parts_json,
+    getenv_as_bool,
+    json_to_seldon_message,
+    seldon_message_to_json,
 )
-from seldon_core.flask_utils import SeldonMicroserviceException
-from seldon_core.metrics import SeldonMetrics
-from seldon_core.metadata import validate_model_metadata, SeldonInvalidMetadataError
-from google.protobuf import json_format
-from seldon_core.proto import prediction_pb2
-from typing import Any, Union, List, Dict
-import numpy as np
 
 logger = logging.getLogger(__name__)
 
@@ -37,28 +50,30 @@ def handle_raw_custom_metrics(
     msg: Union[prediction_pb2.SeldonMessage, Dict],
     seldon_metrics: SeldonMetrics,
     is_proto: bool,
+    method: str,
 ):
     """
     Update SeldonMetrics object with custom metrics from raw methods.
     If INCLUDE_METRICS_IN_CLIENT_RESPONSE environmental variable is set to "true"
     metrics will be dropped from msg.
     """
+    metrics = []
     if is_proto:
         metrics = seldon_message_to_json(msg.meta).get("metrics", [])
         if metrics and not INCLUDE_METRICS_IN_CLIENT_RESPONSE:
             del msg.meta.metrics[:]
-    else:
+    elif isinstance(msg, dict):
         metrics = msg.get("meta", {}).get("metrics", [])
         if metrics and not INCLUDE_METRICS_IN_CLIENT_RESPONSE:
             del msg["meta"]["metrics"]
-    seldon_metrics.update(metrics)
+    seldon_metrics.update(metrics, method)
 
 
 def predict(
     user_model: Any,
-    request: Union[prediction_pb2.SeldonMessage, List, Dict],
+    request: Union[prediction_pb2.SeldonMessage, List, Dict, bytes],
     seldon_metrics: SeldonMetrics,
-) -> Union[prediction_pb2.SeldonMessage, List, Dict]:
+) -> Union[prediction_pb2.SeldonMessage, List, Dict, bytes]:
     """
     Call the user model to get a prediction and package the response
 
@@ -73,6 +88,8 @@ def predict(
     -------
       The prediction
     """
+    # TODO: Find a way to choose predict_rest or predict_grpc when payload is
+    # not decoded
     is_proto = isinstance(request, prediction_pb2.SeldonMessage)
 
     if hasattr(user_model, "predict_rest") and not is_proto:
@@ -85,33 +102,59 @@ def predict(
         if hasattr(user_model, "predict_raw"):
             try:
                 response = user_model.predict_raw(request)
-                handle_raw_custom_metrics(response, seldon_metrics, is_proto)
+                handle_raw_custom_metrics(
+                    response, seldon_metrics, is_proto, PREDICT_METRIC_METHOD_TAG
+                )
                 return response
             except SeldonNotImplementedError:
                 pass
 
         if is_proto:
             (features, meta, datadef, data_type) = extract_request_parts(request)
+
             client_response = client_predict(
                 user_model, features, datadef.names, meta=meta
             )
 
-            metrics = client_custom_metrics(user_model, seldon_metrics)
+            metrics = client_custom_metrics(
+                user_model,
+                seldon_metrics,
+                PREDICT_METRIC_METHOD_TAG,
+                client_response.metrics,
+            )
 
             return construct_response(
-                user_model, False, request, client_response, meta, metrics
+                user_model,
+                False,
+                request,
+                client_response.data,
+                meta,
+                metrics,
+                client_response.tags,
             )
         else:
             (features, meta, datadef, data_type) = extract_request_parts_json(request)
             class_names = datadef["names"] if datadef and "names" in datadef else []
+
             client_response = client_predict(
                 user_model, features, class_names, meta=meta
             )
 
-            metrics = client_custom_metrics(user_model, seldon_metrics)
+            metrics = client_custom_metrics(
+                user_model,
+                seldon_metrics,
+                PREDICT_METRIC_METHOD_TAG,
+                client_response.metrics,
+            )
 
             return construct_response_json(
-                user_model, False, request, client_response, meta, metrics
+                user_model,
+                False,
+                request,
+                client_response.data,
+                meta,
+                metrics,
+                client_response.tags,
             )
 
 
@@ -136,6 +179,7 @@ def send_feedback(
     -------
 
     """
+    is_proto = isinstance(request, prediction_pb2.SeldonMessage)
     seldon_metrics.update_reward(request.reward)
 
     if hasattr(user_model, "send_feedback_rest"):
@@ -150,7 +194,11 @@ def send_feedback(
     else:
         if hasattr(user_model, "send_feedback_raw"):
             try:
-                return user_model.send_feedback_raw(request)
+                response = user_model.send_feedback_raw(request)
+                handle_raw_custom_metrics(
+                    response, seldon_metrics, is_proto, FEEDBACK_METRIC_METHOD_TAG
+                )
+                return response
             except SeldonNotImplementedError:
                 pass
 
@@ -158,15 +206,30 @@ def send_feedback(
             request
         )
         routing = request.response.meta.routing.get(predictive_unit_id)
+
         client_response = client_send_feedback(
             user_model, features, datadef_request.names, reward, truth, routing
         )
 
-        if client_response is None:
-            client_response = np.array([])
-        else:
-            client_response = np.array(client_response)
-        return construct_response(user_model, False, request.request, client_response)
+        metrics = client_custom_metrics(
+            user_model,
+            seldon_metrics,
+            FEEDBACK_METRIC_METHOD_TAG,
+            client_response.metrics,
+        )
+
+        if client_response.data is None:
+            client_response.data = np.array([])
+
+        return construct_response(
+            user_model,
+            False,
+            request.request,
+            client_response.data,
+            None,
+            metrics,
+            client_response.tags,
+        )
 
 
 def transform_input(
@@ -204,33 +267,62 @@ def transform_input(
         if hasattr(user_model, "transform_input_raw"):
             try:
                 response = user_model.transform_input_raw(request)
-                handle_raw_custom_metrics(response, seldon_metrics, is_proto)
+                handle_raw_custom_metrics(
+                    response,
+                    seldon_metrics,
+                    is_proto,
+                    INPUT_TRANSFORM_METRIC_METHOD_TAG,
+                )
                 return response
             except SeldonNotImplementedError:
                 pass
 
         if is_proto:
             (features, meta, datadef, data_type) = extract_request_parts(request)
+
             client_response = client_transform_input(
                 user_model, features, datadef.names, meta=meta
             )
 
-            metrics = client_custom_metrics(user_model, seldon_metrics)
+            metrics = client_custom_metrics(
+                user_model,
+                seldon_metrics,
+                INPUT_TRANSFORM_METRIC_METHOD_TAG,
+                client_response.metrics,
+            )
 
             return construct_response(
-                user_model, False, request, client_response, meta, metrics
+                user_model,
+                False,
+                request,
+                client_response.data,
+                meta,
+                metrics,
+                client_response.tags,
             )
         else:
             (features, meta, datadef, data_type) = extract_request_parts_json(request)
             class_names = datadef["names"] if datadef and "names" in datadef else []
+
             client_response = client_transform_input(
                 user_model, features, class_names, meta=meta
             )
 
-            metrics = client_custom_metrics(user_model, seldon_metrics)
+            metrics = client_custom_metrics(
+                user_model,
+                seldon_metrics,
+                INPUT_TRANSFORM_METRIC_METHOD_TAG,
+                client_response.metrics,
+            )
 
             return construct_response_json(
-                user_model, False, request, client_response, meta, metrics
+                user_model,
+                False,
+                request,
+                client_response.data,
+                meta,
+                metrics,
+                client_response.tags,
             )
 
 
@@ -269,33 +361,62 @@ def transform_output(
         if hasattr(user_model, "transform_output_raw"):
             try:
                 response = user_model.transform_output_raw(request)
-                handle_raw_custom_metrics(response, seldon_metrics, is_proto)
+                handle_raw_custom_metrics(
+                    response,
+                    seldon_metrics,
+                    is_proto,
+                    OUTPUT_TRANSFORM_METRIC_METHOD_TAG,
+                )
                 return response
             except SeldonNotImplementedError:
                 pass
 
         if is_proto:
             (features, meta, datadef, data_type) = extract_request_parts(request)
+
             client_response = client_transform_output(
                 user_model, features, datadef.names, meta=meta
             )
 
-            metrics = client_custom_metrics(user_model, seldon_metrics)
+            metrics = client_custom_metrics(
+                user_model,
+                seldon_metrics,
+                OUTPUT_TRANSFORM_METRIC_METHOD_TAG,
+                client_response.metrics,
+            )
 
             return construct_response(
-                user_model, False, request, client_response, meta, metrics
+                user_model,
+                False,
+                request,
+                client_response.data,
+                meta,
+                metrics,
+                client_response.tags,
             )
         else:
             (features, meta, datadef, data_type) = extract_request_parts_json(request)
             class_names = datadef["names"] if datadef and "names" in datadef else []
+
             client_response = client_transform_output(
                 user_model, features, class_names, meta=meta
             )
 
-            metrics = client_custom_metrics(user_model, seldon_metrics)
+            metrics = client_custom_metrics(
+                user_model,
+                seldon_metrics,
+                OUTPUT_TRANSFORM_METRIC_METHOD_TAG,
+                client_response.metrics,
+            )
 
             return construct_response_json(
-                user_model, False, request, client_response, meta, metrics
+                user_model,
+                False,
+                request,
+                client_response.data,
+                meta,
+                metrics,
+                client_response.tags,
             )
 
 
@@ -305,7 +426,6 @@ def route(
     seldon_metrics: SeldonMetrics,
 ) -> Union[prediction_pb2.SeldonMessage, List, Dict]:
     """
-
     Parameters
     ----------
     user_model
@@ -314,10 +434,8 @@ def route(
        A SelodonMessage proto
     seldon_metrics
         A SeldonMetrics instance
-
     Returns
     -------
-
     """
     is_proto = isinstance(request, prediction_pb2.SeldonMessage)
 
@@ -331,7 +449,9 @@ def route(
         if hasattr(user_model, "route_raw"):
             try:
                 response = user_model.route_raw(request)
-                handle_raw_custom_metrics(response, seldon_metrics, is_proto)
+                handle_raw_custom_metrics(
+                    response, seldon_metrics, is_proto, ROUTER_METRIC_METHOD_TAG
+                )
                 return response
             except SeldonNotImplementedError:
                 pass
@@ -341,31 +461,53 @@ def route(
             client_response = client_route(
                 user_model, features, datadef.names, meta=meta
             )
-            if not isinstance(client_response, int):
+            if not isinstance(client_response.data, int):
                 raise SeldonMicroserviceException(
-                    "Routing response must be int but got " + str(client_response)
+                    "Routing response must be int but got " + str(client_response.data)
                 )
-            client_response_arr = np.array([[client_response]])
+            client_response_arr = np.array([[client_response.data]])
 
-            metrics = client_custom_metrics(user_model, seldon_metrics)
+            metrics = client_custom_metrics(
+                user_model,
+                seldon_metrics,
+                ROUTER_METRIC_METHOD_TAG,
+                client_response.metrics,
+            )
 
             return construct_response(
-                user_model, False, request, client_response_arr, None, metrics
+                user_model,
+                False,
+                request,
+                client_response_arr,
+                None,
+                metrics,
+                client_response.tags,
             )
         else:
             (features, meta, datadef, data_type) = extract_request_parts_json(request)
             class_names = datadef["names"] if datadef and "names" in datadef else []
             client_response = client_route(user_model, features, class_names, meta=meta)
-            if not isinstance(client_response, int):
+            if not isinstance(client_response.data, int):
                 raise SeldonMicroserviceException(
-                    "Routing response must be int but got " + str(client_response)
+                    "Routing response must be int but got " + str(client_response.data)
                 )
-            client_response_arr = np.array([[client_response]])
+            client_response_arr = np.array([[client_response.data]])
 
-            metrics = client_custom_metrics(user_model, seldon_metrics)
+            metrics = client_custom_metrics(
+                user_model,
+                seldon_metrics,
+                ROUTER_METRIC_METHOD_TAG,
+                client_response.metrics,
+            )
 
             return construct_response_json(
-                user_model, False, request, client_response_arr, None, metrics
+                user_model,
+                False,
+                request,
+                client_response_arr,
+                None,
+                metrics,
+                client_response.tags,
             )
 
 
@@ -419,7 +561,9 @@ def aggregate(
         if hasattr(user_model, "aggregate_raw"):
             try:
                 response = user_model.aggregate_raw(request)
-                handle_raw_custom_metrics(response, seldon_metrics, is_proto)
+                handle_raw_custom_metrics(
+                    response, seldon_metrics, is_proto, AGGREGATE_METRIC_METHOD_TAG
+                )
                 return response
             except SeldonNotImplementedError:
                 pass
@@ -437,15 +581,21 @@ def aggregate(
 
             client_response = client_aggregate(user_model, features_list, names_list)
 
-            metrics = client_custom_metrics(user_model, seldon_metrics)
+            metrics = client_custom_metrics(
+                user_model,
+                seldon_metrics,
+                AGGREGATE_METRIC_METHOD_TAG,
+                client_response.metrics,
+            )
 
             return construct_response(
                 user_model,
                 False,
                 request.seldonMessages[0],
-                client_response,
+                client_response.data,
                 merge_meta(meta_list),
                 merge_metrics(meta_list, metrics),
+                client_response.tags,
             )
         else:
             features_list = []
@@ -472,15 +622,21 @@ def aggregate(
 
             client_response = client_aggregate(user_model, features_list, names_list)
 
-            metrics = client_custom_metrics(user_model, seldon_metrics)
+            metrics = client_custom_metrics(
+                user_model,
+                seldon_metrics,
+                AGGREGATE_METRIC_METHOD_TAG,
+                client_response.metrics,
+            )
 
             return construct_response_json(
                 user_model,
                 False,
                 msgs[0],
-                client_response,
+                client_response.data,
                 merge_meta(meta_list),
                 merge_metrics(meta_list, metrics),
+                client_response.tags,
             )
 
 
@@ -509,7 +665,9 @@ def health_status(
             pass
 
     client_response = client_health_status(user_model)
-    metrics = client_custom_metrics(user_model, seldon_metrics)
+    metrics = client_custom_metrics(
+        user_model, seldon_metrics, HEALTH_METRIC_METHOD_TAG
+    )
 
     return construct_response_json(
         user_model, False, {}, client_response, None, metrics
