@@ -1,24 +1,26 @@
+import base64
+import io
 import json
 import logging
+from unittest import mock
+
 import numpy as np
 from google.protobuf import json_format
-import base64
 from PIL import Image
-import io
 
-from seldon_core.wrapper import get_rest_microservice, SeldonModelGRPC, get_grpc_server
+from seldon_core.flask_utils import SeldonMicroserviceException
+from seldon_core.imports_helper import _TF_PRESENT
 from seldon_core.metrics import SeldonMetrics
 from seldon_core.proto import prediction_pb2
 from seldon_core.user_model import SeldonComponent
-from seldon_core.utils import seldon_message_to_json, json_to_seldon_message
-from seldon_core.flask_utils import SeldonMicroserviceException
-from seldon_core.imports_helper import _TF_PRESENT
+from seldon_core.utils import json_to_seldon_message, seldon_message_to_json
+from seldon_core.wrapper import SeldonModelGRPC, get_grpc_server, get_rest_microservice
 
 from .utils import skipif_tf_missing
 
 if _TF_PRESENT:
-    from tensorflow.core.framework.tensor_pb2 import TensorProto
     import tensorflow as tf
+    from tensorflow.core.framework.tensor_pb2 import TensorProto
 
 HEALTH_PING_URL = "/health/ping"
 HEALTH_STATUS_URL = "/health/status"
@@ -50,6 +52,7 @@ class UserObject(SeldonComponent):
         "platform": "model-platform",
         "inputs": [{"name": "input", "datatype": "BYTES", "shape": [1]}],
         "outputs": [{"name": "output", "datatype": "BYTES", "shape": [1]}],
+        "custom": {},
     }
 
     def __init__(self, metrics_ok=True, ret_nparray=False, ret_meta=False):
@@ -308,6 +311,84 @@ def test_model_puid_ok():
     assert j["meta"]["puid"] == "123"
 
 
+@mock.patch("seldon_core.utils.model_name", "my-test-model")
+@mock.patch("seldon_core.utils.image_name", "my-test-model-image")
+def test_requestPath_ok():
+    user_object = UserObject()
+    seldon_metrics = SeldonMetrics()
+    app = get_rest_microservice(user_object, seldon_metrics)
+    client = app.test_client()
+    rv = client.get(
+        '/predict?json={"meta":{"puid":"123"},"data":{"names":["a","b"],"ndarray":[[1,2]]}}'
+    )
+    j = json.loads(rv.data)
+    logging.info(j)
+    assert rv.status_code == 200
+    assert j["meta"]["requestPath"] == {"my-test-model": "my-test-model-image"}
+
+
+@mock.patch("seldon_core.utils.model_name", "my-test-model")
+@mock.patch("seldon_core.utils.image_name", "my-test-model-image")
+def test_requestPath_2nd_node_ok():
+    user_object = UserObject()
+    seldon_metrics = SeldonMetrics()
+    app = get_rest_microservice(user_object, seldon_metrics)
+    client = app.test_client()
+    rv = client.get(
+        '/predict?json={"meta":{"requestPath":{"earlier-node": "earlier-image"}},"data":{"names":["a","b"],"ndarray":[[1,2]]}}'
+    )
+    j = json.loads(rv.data)
+    logging.info(j)
+    assert rv.status_code == 200
+    assert j["meta"]["requestPath"] == {
+        "my-test-model": "my-test-model-image",
+        "earlier-node": "earlier-image",
+    }
+
+
+@mock.patch("seldon_core.utils.model_name", "my-test-model")
+@mock.patch("seldon_core.utils.image_name", "my-test-model-image")
+def test_proto_requestPath_ok():
+    user_object = UserObject()
+    seldon_metrics = SeldonMetrics()
+    app = SeldonModelGRPC(user_object, seldon_metrics)
+    arr = np.array([1, 2])
+    datadef = prediction_pb2.DefaultData(
+        tensor=prediction_pb2.Tensor(shape=(2, 1), values=arr)
+    )
+    meta = prediction_pb2.Meta()
+    json_format.ParseDict({"tags": {"foo": "bar"}}, meta)
+    request = prediction_pb2.SeldonMessage(data=datadef, meta=meta)
+    resp = app.Predict(request, None)
+    jStr = json_format.MessageToJson(resp)
+    j = json.loads(jStr)
+    logging.info(j)
+    assert j["meta"]["requestPath"] == {"my-test-model": "my-test-model-image"}
+
+
+@mock.patch("seldon_core.utils.model_name", "my-test-model")
+@mock.patch("seldon_core.utils.image_name", "my-test-model-image")
+def test_proto_requestPath_2nd_node_ok():
+    user_object = UserObject()
+    seldon_metrics = SeldonMetrics()
+    app = SeldonModelGRPC(user_object, seldon_metrics)
+    arr = np.array([1, 2])
+    datadef = prediction_pb2.DefaultData(
+        tensor=prediction_pb2.Tensor(shape=(2, 1), values=arr)
+    )
+    meta = prediction_pb2.Meta()
+    json_format.ParseDict({"requestPath": {"earlier-node": "earlier-image"}}, meta)
+    request = prediction_pb2.SeldonMessage(data=datadef, meta=meta)
+    resp = app.Predict(request, None)
+    jStr = json_format.MessageToJson(resp)
+    j = json.loads(jStr)
+    logging.info(j)
+    assert j["meta"]["requestPath"] == {
+        "my-test-model": "my-test-model-image",
+        "earlier-node": "earlier-image",
+    }
+
+
 def test_model_lowlevel_ok():
     user_object = UserObjectLowLevel()
     seldon_metrics = SeldonMetrics()
@@ -943,7 +1024,7 @@ def test_unimplemented_predict_raw_on_seldon_component():
 
 
 def test_unimplemented_predict_raw():
-    class CustomObject(object):
+    class CustomObject:
         def predict(self, X, features_names, **kwargs):
             return X * 2
 
@@ -978,7 +1059,7 @@ def test_unimplemented_feedback_raw_on_seldon_component():
 
 
 def test_unimplemented_feedback_raw():
-    class CustomObject(object):
+    class CustomObject:
         def feedback(self, features, feature_names, reward, truth):
             logging.info("Feedback called")
 
